1,999bd
e Makefile.PL
execute "edit" glob("`find Devel-Cover\* -name MANIFEST | tail -1`")
e Changes
e docs/TODO
e docs/BUGS
e lib/Devel/Cover.pm
e lib/Devel/Cover/DB.pm
e Cover.xs
e bin/cover
e bin/cpancover
e lib/Devel/Cover/Test.pm
e lib/Devel/Cover/Report/Text.pm
e lib/Devel/Cover/Report/Html_minimal.pm
e lib/Devel/Cover/Report/Html_basic.pm
e #1
Tlist
